import {inject, singleton} from 'tsyringe'
import {RingtoneService} from './service'
import {DoubleDeletionError} from './response'

@singleton()
export class RingtoneController {
	constructor(
		@inject('RingtoneService')
		private readonly ringtoneService: RingtoneService
	) {
	}

	get = async (request: any, reply: any) => {
		try {
			const data = await this.ringtoneService.get(request.params['id'])
			if (typeof (data.deletedAt as Date) !== 'undefined') {
				reply.gone()
				return
			}
			if (!data) {
				reply.notFound()
				return
			}
			return {
				data,
			}
		} catch (err) {
			reply.internalServerError(err.message)
		}
	}

	browse = async (request: any, reply: any) => {
		try {
			// TODO deserialize query
			const { 'skip': skipRaw, 'take': takeRaw } = request.query
			const skipNumber = Number(skipRaw)
			const takeNumber = Number(takeRaw)

			const skip = !isNaN(skipNumber) ? skipNumber : undefined
			const take = !isNaN(takeNumber) ? takeNumber : undefined

			return {
				data: await this.ringtoneService.browse(skip, take),
				skip,
				take,
			}
		} catch (err) {
			reply.internalServerError(err.message)
		}
	}

	search = async (request: any, reply: any) => {
		try {
			const { 'q': query } = request.query
			return {
				data: await this.ringtoneService.search(query),
				query,
			}
		} catch (err) {
			reply.internalServerError(err.message)
		}
	}

	create = async (request: any, reply: any) => {
		try {
			const data = await this.ringtoneService.create(request.body)
			reply.status(201)
			return {
				data,
			}
		} catch (err) {
			reply.internalServerError(err.message)
		}
	}

	update = async (request: any, reply: any) => {
		try {
			// TODO validate data
			const data = await this.ringtoneService.update({
				...request.body,
				id: request.params['id'],
			})
			if (data.deletedAt !== null) {
				reply.gone()
				return
			}
			if (!data) {
				reply.notFound()
				return
			}
			return {
				data,
			}
		} catch (err) {
			reply.internalServerError(err.message)
		}
	}

	softDelete = async (request: any, reply: any) => {
		try {
			// TODO validate data
			const data = await this.ringtoneService.softDelete(request.params['id'])
			if (!data) {
				reply.notFound()
				return
			}
			return {
				data,
			}
		} catch (err) {
			reply.internalServerError(err.message)
		}
	}

	undoDelete = async (request: any, reply: any) => {
		try {
			// TODO validate data
			const data = await this.ringtoneService.undoDelete(request.params['id'])
			if (!data) {
				reply.notFound()
				return
			}
			return {
				data,
			}
		} catch (err) {
			reply.internalServerError(err.message)
		}
	}

	hardDelete = async (request: any, reply: any) => {
		try {
			// TODO validate data
			await this.ringtoneService.hardDelete(request.params['id'])
			reply.status(204)
		} catch (err) {
			if (err instanceof DoubleDeletionError) {
				reply.notFound(err.message)
			}
			reply.internalServerError(err.message)
		}
	}
}
